package cn.edu.xidian.graduation.modelchecking;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import cn.edu.xidian.graduation.modelchecking.model.AbstractState;
import cn.edu.xidian.graduation.modelchecking.model.Conterexample;
import cn.edu.xidian.graduation.modelchecking.model.Model;
import cn.edu.xidian.graduation.modelchecking.model.OriginalState;
import cn.edu.xidian.graduation.modelchecking.model.State;
import cn.edu.xidian.graduation.modelchecking.util.ConterexampleGenerator;
import cn.edu.xidian.graduation.modelchecking.util.ModelGenerator;
import cn.edu.xidian.graduation.modelchecking.util.ModelOutputer;
import cn.edu.xidian.graduation.modelchecking.util.SpuriousChecker;

public class ModelChecker {
	public static final int STATE_SIZE = 150;
	public static final long RALATION_SIZE = 5000;

	public static final int CONTEREXAMPLE_SIZE_MIN = 150;
	public static final int CONTEREXAMPLE_SIZE_MAX =150;

	private static Map<String, Object[]> keyAndValues() {
		Map<String, Object[]> kv = new HashMap<String, Object[]>();
		Integer[] x = new Integer[30];
		for (int i = 0; i < x.length; i++) {
			x[i] = i;
		}

		Integer[] y = new Integer[4];
		for (int i = 0; i < y.length; i++) {
			y[i] = i + 2;
		}

		kv.put("x", x);
		kv.put("y", y);
		return kv;
	}

	public static void main(String[] args) throws InterruptedException {
		// key and values
		Map<String, Object[]> kv = keyAndValues();

		// original model
		Model originalModel = ModelGenerator.randomOriginalModel(STATE_SIZE,
				RALATION_SIZE, kv);

		// abstract model
		Model abstractModel = ModelGenerator.abstarct(originalModel, "y");
		System.out.println("stateSize:" + abstractModel.stateSize()
				+ ",relationSize:" + abstractModel.relationSize());

		/* write these models to image files */
		//outputAsImage(originalModel, abstractModel);

		// conterexample
		Conterexample conterexample = ConterexampleGenerator
				.randomConterexample(abstractModel, CONTEREXAMPLE_SIZE_MIN,
						CONTEREXAMPLE_SIZE_MAX);
		System.out.println(conterexample);

		setStateWeights(conterexample, originalModel);

		doCheck(originalModel, abstractModel, conterexample);
	}

	private static void doCheck(Model originalModel, Model abstractModel,
			Conterexample conterexample) throws InterruptedException {

		// System.out.println(abstractModel);

		// print this conterexample to console
		// System.out.println("size:" + conterexample.size() + ", "
		// + conterexample);

		System.out.println("checkSpurious1 starts...");
		long time1 = System.currentTimeMillis();
		AbstractState failureState1 = SpuriousChecker.checkSpurious1(
				originalModel, abstractModel, conterexample);
		System.out.println(failureState1);
		System.out.println("checkSpurious1 ends,total time:"
				+ (System.currentTimeMillis() - time1) + "ms");

		printLineSeperator();

		System.out.println("checkSpurious2 starts...");
		long time2 = System.currentTimeMillis();
		AbstractState failureState2 = SpuriousChecker.checkSpurious2(
				originalModel, abstractModel, conterexample);
		System.out.println(failureState2);
		System.out.println("checkSpurious2 ends,total time:"
				+ (System.currentTimeMillis() - time2) + "ms");

		printLineSeperator();

		System.out.println("checkSpurious3 starts...");
		long time3 = System.currentTimeMillis();
		AbstractState failureState3 = SpuriousChecker.checkSpurious3(
				originalModel, abstractModel, conterexample);
		System.out.println(failureState3);
		System.out.println("checkSpurious3 ends,total time:"
				+ (System.currentTimeMillis() - time3) + "ms");

		printLineSeperator();

		System.out.println("checkSpurious4 starts...");
		long time4 = System.currentTimeMillis();
		AbstractState failureState4 = SpuriousChecker.checkSpurious4(
				originalModel, abstractModel, conterexample);
		System.out.println(failureState4);
		System.out.println("checkSpurious4 ends,total time:"
				+ (System.currentTimeMillis() - time4) + "ms");

		printLineSeperator();

		System.out.println("splitPath starts...");
		long time5 = System.currentTimeMillis();
		AbstractState failureState5 = SpuriousChecker
				.checkSpuriousWithSplithPath(originalModel, abstractModel,
						conterexample);
		System.out.println(failureState5);
		System.out.println("splitpath ends,total time:"
				+ (System.currentTimeMillis() - time5) + "ms");

	}

	private static void setStateWeights(Conterexample conterexample,
			Model originalModel) throws InterruptedException {
		List<AbstractState> states = conterexample.getStates();
		if (states != null) {
			int size = states.size();
			if (size > 0) {
				int poolSize = 100;
				if (size < poolSize) {
					poolSize = size;
				}
				ExecutorService executor = Executors
						.newFixedThreadPool(poolSize);
				for (int i = 0; i < size; i++) {
					final AbstractState abstractState = states.get(i);
					executor.execute(new Task(abstractState, originalModel));
				}
				executor.shutdown();
				executor.awaitTermination(1, TimeUnit.HOURS);
			}

		}

	}

	private static class Task implements Runnable {
		private AbstractState abstractState;
		private Model originalModel;

		public Task(AbstractState abstractState, Model originalModel) {
			super();
			this.abstractState = abstractState;
			this.originalModel = originalModel;
		}

		@Override
		public void run() {
			long weight = (long) ein(abstractState, originalModel)
					* eout(abstractState, originalModel);
			abstractState.setWeight(weight);

		}

	}

	private static int ein(AbstractState abstractState, Model originalModel) {
		Set<OriginalState> states = abstractState.getData();
		int result = 0;
		for (OriginalState state : states) {
			Set<State> precusor = originalModel.getPrecursor(state);
			if (precusor != null) {
				precusor.removeAll(states);
				result += precusor.size();
			}
		}
		return result;
	}

	private static int eout(AbstractState abstractState, Model originalModel) {
		Set<OriginalState> states = abstractState.getData();
		int result = 0;
		for (OriginalState state : states) {
			Set<State> successor = originalModel.getSuccessor(state);
			if (successor != null) {
				successor.removeAll(states);
				result += successor.size();
			}
		}
		return result;
	}

	private static void outputAsImage(Model originalModel, Model abstractModel) {
		ModelOutputer.output(originalModel, ModelOutputer.ORIGINAL_MODEL);
		// ModelOutputer
		// .output(abstractModel, ModelOutputer.SIMPLE_ABSTRACT_MODEL);
		ModelOutputer.outputAbstractModel(originalModel, abstractModel);
	}

	private static void printLineSeperator() {
		System.out
				.println("-----------------------------------------------------------");
	}

}
