package cn.edu.xidian.graduation.modelchecking.model;

import java.util.Set;

/**
 * A model is actually a graph which holds all the states and their relations
 * 
 * @author wenqiang.fanwq
 * 
 */
public interface Model {
	/**
	 * Add a state to this model just like add a vertex to a graph.
	 * 
	 * @param state
	 *            the state to be added to this model
	 */
	void addState(State state);

	Set<State> getAllStates();

	/**
	 * Just like add a edge to a graph
	 * 
	 * @param beg
	 *            the state that this relation begins with
	 * @param end
	 *            the state that this relation ends with
	 */
	void addRelation(State beg, State end);

	boolean containsState(State state);

	/**
	 * Indicate whether there is a relation between these states.
	 * 
	 * @return true if a relation begins with beg and ends with end
	 */
	boolean hasRelation(State beg, State end);

	Set<State> getSuccessor(State state);

	Set<State> getPrecursor(State state);

	boolean hasSuccessor(State state);

	boolean hasPrecursor(State state);

	Set<State> getInitStates();

	/**
	 * @return all states that don't have successors
	 */
	Set<State> allNoSuccessorStates();

	/**
	 * @return all states that don't have percursors
	 */
	Set<State> allNoPrecusorStaes();

	boolean isEmpty();

	int stateSize();

	long relationSize();
	
}
