package cn.edu.xidian.graduation.modelchecking.util;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

/**
 * This utility class provides some static methods for collection operation
 * 
 * @author wenqiang.fanwq
 * 
 */
public class CollectionUtil {
	/**
	 * Retrieve the intersection of the given two sets. If the parameters are
	 * HashSet, the algorithm's time complexity will be O(min(set1.size(),
	 * set2.size())).
	 * 
	 * @return the intersection of the given two sets
	 */
	public static <T> Set<T> intersection(Set<T> set1, Set<T> set2) {
		if (set1 == null || set2 == null) {
			return null;
		}
		if (set1.size() > set2.size()) {
			Set<T> temp = set1;
			set1 = set2;
			set2 = temp;
		}

		Set<T> result = new HashSet<T>();
		Iterator<T> iter = set1.iterator();
		while (iter.hasNext()) {
			T state = iter.next();
			if (set2.contains(state)) {
				result.add(state);
			}
		}
		return result;
	}

	/**
	 * Retrieve two sets's union, this method simply use the addAll method
	 * defined in Set interface to obtain the result.
	 * 
	 * @return
	 */
	public static <T> Set<T> union(Set<T> set1, Set<T> set2) {
		if (set1 == null) {
			return set2;
		}
		if (set2 == null) {
			return set1;
		}
		if (set1.size() < set2.size()) {
			Set<T> temp = set1;
			set1 = set2;
			set2 = temp;
		}

		set1.addAll(set2);
		return set1;
	}

	/**
	 * @return ture if the given set is null or its size is 0
	 */
	public static boolean isEmptySet(Set set) {
		return set == null || set.size() == 0;
	}

	/**
	 * This method simply return a list that contains all the elements in the
	 * given set
	 * 
	 * @param set
	 *            the given set
	 * @return a list represent all the elements in the given set
	 */
	public static <T> List<T> list(Set<T> set) {
		if (set == null) {
			return null;
		}
		Iterator<T> iter = set.iterator();
		List<T> list = new ArrayList<T>();
		while (iter.hasNext()) {
			list.add(iter.next());
		}
		return list;
	}

	/**
	 * @return a random integer number between 0 and max
	 */
	public static int randomInteger(int max) {
		return (int) (Math.random() * max);
	}

	/*
	 * for test
	 */
	public static void main(String[] args) {
		Set<Integer> set = new HashSet<Integer>();
		set.add(1);
		set.add(2);
		System.out.println(list(set));
	}
}
