package cn.edu.xidian.graduation.modelchecking.util;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import cn.edu.xidian.graduation.modelchecking.model.AbstractState;
import cn.edu.xidian.graduation.modelchecking.model.Conterexample;
import cn.edu.xidian.graduation.modelchecking.model.Model;
import cn.edu.xidian.graduation.modelchecking.model.State;

public class ConterexampleGenerator {
	private static final int THRESHHOLD = 10;

	public static Conterexample randomConterexample(Model abstractModel,
			int minSize, int maxSize) {
		if (abstractModel == null) {
			throw new IllegalArgumentException(
					"abstract model must not be null");
		}
		if (minSize > maxSize) {
			throw new IllegalArgumentException(
					"the minSize is greater than maxSize, please check your input");
		}

		Conterexample max = null;
		for (int i = 0; i < THRESHHOLD; i++) {
			Conterexample ce = process(abstractModel, maxSize);
			if (ce.size() >= minSize) {
				return ce;
			}

			if (max == null || max.size() < ce.size()) {
				max = ce;
			}
		}
		return max;
	}

	private static Conterexample process(Model abstractModel, int maxSize) {
		Set<State> states = abstractModel.getAllStates();
		List<State> statesList = CollectionUtil.list(states);
		int randomIndex = CollectionUtil.randomInteger(statesList.size());
		AbstractState state = (AbstractState) statesList.get(randomIndex);

		List<AbstractState> abstractStates = new ArrayList<AbstractState>();
		abstractStates.add(state);

		int count = 1;
		while (abstractModel.hasSuccessor(state) && count < maxSize) {
			state = randomSuccessor(state, abstractModel);
			if (state != null) {
				abstractStates.add(state);
				++count;
			}
		}

		Conterexample conterexample = new Conterexample();
		conterexample.setStates(abstractStates);
		return conterexample;
	}

	private static AbstractState randomSuccessor(AbstractState abstractState,
			Model abstractModel) {
		Set<State> successors = abstractModel.getSuccessor(abstractState);
		List<State> statesList = CollectionUtil.list(successors);
		if(statesList==null || statesList.size()==0){
			return null;
		}

		int index = CollectionUtil.randomInteger(statesList.size());
		return (AbstractState) statesList.get(index);

	}
}
