package cn.edu.xidian.graduation.modelchecking.util;

import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.CompletionService;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

import cn.edu.xidian.graduation.modelchecking.model.AbstractState;
import cn.edu.xidian.graduation.modelchecking.model.Model;
import cn.edu.xidian.graduation.modelchecking.model.OriginalState;
import cn.edu.xidian.graduation.modelchecking.model.SimpleModel;
import cn.edu.xidian.graduation.modelchecking.model.State;

/**
 * This util class use to create a Model through files or program
 * 
 * @author wenqiang.fanwq
 * 
 */
public class ModelGenerator {
	public static Model readJson(String file) {
		// TODO
		return null;
	}

	public static Model readXml(String file) {
		// TODO
		return null;
	}

	/**
	 * 
	 * @param stateSize
	 *            the size of states this original model will have
	 * @param relationSize
	 *            the size of edges this original model will have
	 * @param keyAndValues
	 *            this map provides the OriginalState's data value. the
	 *            value,the Object array, are values the key can hold
	 * @return an original model
	 */
	public static Model randomOriginalModel(int stateSize, long relationSize,
			Map<String, Object[]> keyAndValues) {
		if (stateSize == 0L) {
			return new SimpleModel();
		}
		Model originalModel = new SimpleModel();
		for (int i = 0; i < stateSize; i++) {
			State state = new OriginalState(i);
			Map<String, Object> data = new HashMap<String, Object>(
					(int) (1.75 * keyAndValues.size()));
			for (String key : keyAndValues.keySet()) {
				Object[] values = keyAndValues.get(key);
				Object value = values[CollectionUtil
						.randomInteger(values.length)];
				data.put(key, value);
			}
			state.setData(data);
			originalModel.addState(state);
		}
		// add relatons
		long maxSize = (long) stateSize * stateSize;
		if (relationSize > maxSize) {
			relationSize = maxSize;
		}

		Set<State> states = originalModel.getAllStates();
		List<State> statesList = CollectionUtil.list(states);
		for (long i = 0; i < relationSize; i++) {
			while (true) {
				State beg = statesList.get(CollectionUtil
						.randomInteger(stateSize));
				State end = statesList.get(CollectionUtil
						.randomInteger(stateSize));
				if (!originalModel.hasRelation(beg, end)) {
					originalModel.addRelation(beg, end);
					break;
				}
			}
		}
		return originalModel;

	}

	/**
	 * This is the abstraction method which performs the abstraction by
	 * selecting a set of variables which are insensitive to the desired
	 * property to be invisible and obtain a abstract model
	 * 
	 * @param originalModel
	 * @param invisibleKeys
	 *            this keys will not be used in the process of abstraction
	 * @return a abstract model obtained by this abstraction method
	 * @throws InterruptedException
	 */
	public static Model abstarct(final Model originalModel,
			String... invisibleKeys) throws InterruptedException {
		Set<State> states = originalModel.getAllStates();
		// obatin the list view of states
		List<State> statesList = CollectionUtil.list(states);

		// remove invisible keys which have nothing to do with the abstraction
		// process
		for (State state : statesList) {
			Map<String, Object> data = (Map<String, Object>) state.getData();
			for (String key : invisibleKeys) {
				data.remove(key);
			}
		}

		/*-------------------obtain abstract states------------------------------*/
		// sort
		Collections.sort(statesList, new StateComparator());
		// add a sentry
		statesList.add(new OriginalState(-1));

		final Model abstarctModel = new SimpleModel();
		long id = 0;
		Set<State> originalStates = new HashSet<State>();
		for (int i = 0; i < statesList.size() - 1; i++) {
			originalStates.add(statesList.get(i));
			if (!statesList.get(i).getData()
					.equals(statesList.get(i + 1).getData())) {
				State abstractState = new AbstractState(id++);
				abstractState.setData(originalStates);
				abstarctModel.addState(abstractState);
				// for storing another abstract state's data
				originalStates = new HashSet<State>();
			}
		}

		// thread pool
		ExecutorService executor = Executors.newFixedThreadPool(10);
		/*-----------------and relations ----------------------*/
		Set<State> abstarctStates = abstarctModel.getAllStates();
		for (final State abstractState1 : abstarctStates) { // abstract
															// state1
			for (final State abstractState2 : abstarctStates) {// abstract
																// state2
				final Set<OriginalState> originalStates1 = (Set<OriginalState>) abstractState1
						.getData();
				final Set<OriginalState> originalStates2 = (Set<OriginalState>) abstractState2
						.getData();

				// use a thread to do the task
				executor.execute(new Runnable() {
					@Override
					public void run() {
						next: for (OriginalState originalState1 : originalStates1) {
							for (OriginalState originalState2 : originalStates2) {
								if (originalModel.hasRelation(originalState1,
										originalState2)) {
									abstarctModel.addRelation(abstractState1,
											abstractState2);
									break next;
								}
							}
						}
					}
				});

			}
		}
		executor.shutdown();
		executor.awaitTermination(1, TimeUnit.HOURS);
		return abstarctModel;
	}

	/**
	 * This class provides a comparison method to compare two OriginalState
	 * through their data field.
	 * 
	 * @author wenqiang.fanwq
	 * 
	 */
	private static class StateComparator implements Comparator<State> {

		@Override
		public int compare(State o1, State o2) {
			if (o1 == null) {
				return -1;
			}
			if (o2 == null) {
				return 1;
			}

			Map<String, Comparable> data1 = (Map<String, Comparable>) o1
					.getData();
			Map<String, Comparable> data2 = (Map<String, Comparable>) o2
					.getData();
			Set<String> keys = data1.keySet();
			for (String key : keys) {
				Comparable value1 = data1.get(key);
				Comparable value2 = data2.get(key);
				if (value1 == null) {
					return -1;
				}
				if (value2 == null) {
					return 1;
				}
				int compare = value1.compareTo(value2);
				if (compare != 0) {
					return compare;
				}
			}
			return 0;
		}

	}

	/*
	 * for test
	 */
	/*
	 * public static void main(String[] args) { long time =
	 * System.currentTimeMillis(); Map map = new HashMap(); map.put("x", new
	 * String[] { "1", "2", "3", "4", "5", "6" }); map.put("y", new String[] {
	 * "7", "8", "9", "10", "12", "13" }); Model model = randomOriginalModel(5,
	 * 9, map);
	 * 
	 * System.out.println(model);
	 * System.out.println("-----------------------------"); System.out
	 * .println("cost:" + (System.currentTimeMillis() - time) + "ms");
	 * 
	 * System.out.println(model); abstarct(model, "y");
	 * 
	 * }
	 */
}
