package cn.edu.xidian.graduation.modelchecking.util;

import java.util.List;
import java.util.Set;

import cn.edu.xidian.graduation.modelchecking.graphviz.GraphViz;
import cn.edu.xidian.graduation.modelchecking.model.AbstractState;
import cn.edu.xidian.graduation.modelchecking.model.Model;
import cn.edu.xidian.graduation.modelchecking.model.State;

public class ModelOutputer {
	public static final String TYPE_PNG = ".png";
	public static final String ORIGINAL_MODEL = "original_model" + TYPE_PNG;
	public static final String ABSTRACT_MODEL = "abstract_model" + TYPE_PNG;
	public static final String SIMPLE_ABSTRACT_MODEL = "simple_abstract_model"
			+ TYPE_PNG;

	public static final String SUB_GRAPH = "subgraph cluster";
	public static final String LABEL_EQ = "label = ";

	public static final String INIT_STATE_COLOR_ORIGIN = " [color = tomato]";
	public static final String INIT_STATE_COLOR_ABSTRACTION = " [color = tan]";

	public static void output(Model originalModel, String fileName) {
		GraphViz gv = new GraphViz();
		// dot start
		gv.addln(gv.start_graph());

		Set<State> states = originalModel.getAllStates();
		addEdge(states, gv, originalModel);
		if (fileName.equals(ORIGINAL_MODEL)) {
			addInitStateColor(gv, originalModel, INIT_STATE_COLOR_ORIGIN);
		}

		// dot end
		gv.addln(gv.end_graph());

		// System.out.println(gv.getDotSource());

		// write this graph to a image
		gv.writeGraphToFile(gv.getGraph(gv.getDotSource(), "png"), "resource/"
				+ fileName);
	}

	public static void outputAbstractModel(Model originalModel,
			Model abstractModel) {
		Set<State> abstractStates = abstractModel.getAllStates();

		GraphViz gv = new GraphViz();
		// dot start
		gv.addln(gv.start_graph());
		// gv.addln("compound=true;");
		// gv.addln("concentrate=true;");
		gv.addln();

		List<State> statesList = CollectionUtil.list(abstractStates);
		for (int i = 0; i < statesList.size(); i++) {
			AbstractState s = (AbstractState) statesList.get(i);
			gv.addln(SUB_GRAPH + i + "{");
			gv.addln(LABEL_EQ + s.simpleToString());

			Set<State> states = SpuriousChecker.transform(s.getData());

			for (State st : states) {
				gv.addln(st.simpleToString() + ";");
			}

			gv.addln("}");
			gv.addln();
		}

		addEdge(originalModel.getAllStates(), gv, originalModel);
		addInitStateColor(gv, originalModel, INIT_STATE_COLOR_ORIGIN);

		// dot end
		gv.addln(gv.end_graph());

		gv.writeGraphToFile(gv.getGraph(gv.getDotSource(), "png"), "resource/"
				+ ABSTRACT_MODEL);
	}

	private static void addEdge(Set<State> states, GraphViz gv,
			Model originalModel) {
		for (State state : states) {
			Set<State> suc = originalModel.getSuccessor(state);
			if (CollectionUtil.isEmptySet(suc)) {
				gv.addln(state.simpleToString() + ";");
				continue;
			}
			for (State s : suc) {
				gv.addln(state.simpleToString() + "->" + s.simpleToString()
						+ ";");
			}
		}

	}

	private static void addInitStateColor(GraphViz gv, Model model, String color) {
		Set<State> init = model.getInitStates();
		for (State i : init) {
			gv.addln(i.simpleToString() + color + ";");
		}
	}
}
